# Copyright 2017 Square, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import psutil

import errno
import tempfile
import os


class JLock(object):
    """Lockfile for accessing a particular J-Link.

    The J-Link SDK does not prevent accessing the same J-Link multiple times
    from the same process or multiple processes.  As a result, a user can
    have the same J-Link being accessed by multiple processes.  This class
    provides an interface to a lock-file like structure for the physical
    J-Links to ensure that any instance of a ``JLink`` with an open emulator
    connection will be the only one accessing that emulator.

    This class uses a PID-style lockfile to allow acquiring of the lockfile in
    the instances where the lockfile exists, but the process which created it
    is no longer running.

    To share the same emulator connection between multiple threads, processes,
    or functions, a single instance of a ``JLink`` should be created and passed
    between the threads and processes.

    Attributes:
      name: the name of the lockfile.
      path: full path to the lockfile.
      fd: file description of the lockfile.
      acquired: boolean indicating if the lockfile lock has been acquired.
    """

    SERIAL_NAME_FMT = '.pylink-usb-{}.lck'
    IPADDR_NAME_FMT = '.pylink-ip-{}.lck'

    def __init__(self, serial_no):
        """Creates an instance of a ``JLock`` and populates the name.

        Note:
          This method may fail if there is no temporary directory in which to
          have the lockfile placed.

        Args:
          self (JLock): the ``JLock`` instance
          serial_no (int): the serial number of the J-Link

        Returns:
          ``None``
        """
        self.name = self.SERIAL_NAME_FMT.format(serial_no)
        self.acquired = False
        self.fd = None
        self.path = None
        self.path = os.path.join(tempfile.gettempdir(), self.name)

    def __del__(self):
        """Cleans up the lockfile instance if it was acquired.

        Args:
          self (JLock): the ``JLock`` instance

        Returns:
          ``None``
        """
        self.release()

    def acquire(self):
        """Attempts to acquire a lock for the J-Link lockfile.

        If the lockfile exists but does not correspond to an active process,
        the lockfile is first removed, before an attempt is made to acquire it.

        Args:
          self (Jlock): the ``JLock`` instance

        Returns:
          ``True`` if the lock was acquired, otherwise ``False``.

        Raises:
          OSError: on file errors.
        """
        if os.path.exists(self.path):
            try:
                pid = None

                with open(self.path, 'r') as f:
                    line = f.readline().strip()
                    pid = int(line)

                # In the case that the lockfile exists, but the pid does not
                # correspond to a valid process, remove the file.
                if not psutil.pid_exists(pid):
                    os.remove(self.path)

            except ValueError as e:
                # Pidfile is invalid, so just delete it.
                os.remove(self.path)

            except IOError as e:
                # Something happened while trying to read/remove the file, so
                # skip trying to read/remove it.
                pass

        try:
            self.fd = os.open(self.path, os.O_CREAT | os.O_EXCL | os.O_RDWR)

            # PID is written to the file, so that if a process exits wtihout
            # cleaning up the lockfile, we can still acquire the lock.
            to_write = '%s%s' % (os.getpid(), os.linesep)
            os.write(self.fd, to_write.encode())

        except OSError as e:
            if not os.path.exists(self.path):
                raise
            return False

        self.acquired = True
        return True

    def release(self):
        """Cleans up the lockfile if it was acquired.

        Args:
          self (JLock): the ``JLock`` instance

        Returns:
          ``False`` if the lock was not released or the lock is not acquired,
          otherwise ``True``.
        """
        if not self.acquired:
            return False

        os.close(self.fd)

        if os.path.exists(self.path):
            os.remove(self.path)

        self.acquired = False
        return True



# Copyright 2017 Square, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pylink.binpacker as binpacker

import unittest


class TestBinpacker(unittest.TestCase):
    """Unit test class for the `binpacker` submodule."""

    def setUp(self):
        """Called before each test.

        Performs setup.

        Args:
          self (TestBinpacker): the `TestBinpacker` instance

        Returns:
          `None`
        """
        pass

    def tearDown(self):
        """Called after each test.

        Performs teardown.

        Args:
          self (TestBinpacker): the `TestBinpacker` instance

        Returns:
          `None`
        """
        pass

    def test_pack_size_valid(self):
        """Tests the `pack_size()` method.

        Tests that the `pack_size()` method returns the appropriate number of
        bytes based on the value of the integer being passed.

        Args:
          self (TestBinpacker): the `TestBinpacker` instance

        Returns:
          `None`
        """
        self.assertEqual(1, binpacker.pack_size(0))
        self.assertEqual(1, binpacker.pack_size(255))
        self.assertEqual(2, binpacker.pack_size(256))
        self.assertEqual(2, binpacker.pack_size(65535))
        self.assertEqual(3, binpacker.pack_size(65536))
        self.assertEqual(4, binpacker.pack_size(2147483647))
        self.assertEqual(8, binpacker.pack_size(9223372036854775807))

    def test_pack_size_invalid(self):
        """Tests that the `pack_size()` method throws an exception.

        We cannot handle non-strings or negative numbers, so `pack_size()`
        should throw an exception.

        Args:
          self (TestBinpacker): the `TestBinpacker` instance

        Returns:
          `None`
        """
        with self.assertRaises(TypeError):
            binpacker.pack_size('Bender Bending Rodriguez')

        with self.assertRaises(ValueError):
            binpacker.pack_size(-1)

    def test_pack(self):
        """Tests that the `pack()` method packs values explicitly.

        Args:
          self (TestBinpacker): the `TestBinpacker` instance

        Returns:
          `None`
        """
        packed = binpacker.pack(255, 1)
        self.assertEqual(1, len(packed))
        self.assertEqual(255, int(packed[0]))

        packed = binpacker.pack(255, 8)
        self.assertEqual(1, len(packed))
        self.assertEqual(255, int(packed[0]))

        packed = binpacker.pack(255, 16)
        self.assertEqual(2, len(packed))
        self.assertEqual(255, int(packed[0]))
        self.assertEqual(0, int(packed[1]))

        packed = binpacker.pack(255, 32)
        self.assertEqual(4, len(packed))
        self.assertEqual(255, int(packed[0]))
        self.assertEqual(0, int(packed[1]))
        self.assertEqual(0, int(packed[2]))
        self.assertEqual(0, int(packed[3]))

        packed = binpacker.pack(0xFF000000, 32)
        self.assertEqual(4, len(packed))
        self.assertEqual(0, int(packed[0]))
        self.assertEqual(0, int(packed[1]))
        self.assertEqual(0, int(packed[2]))
        self.assertEqual(255, int(packed[3]))

    def test_pack_infer_size(self):
        """Tests that the `pack()` method can pack with an inferred size.

        Args:
          self (TestBinpacker): the `TestBinpacker` instance

        Returns:
          `None`
        """
        packed = binpacker.pack(255)
        self.assertEqual(1, len(packed))
        self.assertEqual(255, int(packed[0]))

        packed = binpacker.pack(256)
        self.assertEqual(2, len(packed))
        self.assertEqual(0, int(packed[0]))
        self.assertEqual(1, int(packed[1]))

        packed = binpacker.pack(0xFF000000)
        self.assertEqual(4, len(packed))
        self.assertEqual(0, int(packed[0]))
        self.assertEqual(0, int(packed[1]))
        self.assertEqual(0, int(packed[2]))
        self.assertEqual(255, int(packed[3]))

    def test_pack_invalid(self):
        """Tests that the `pack()` method raises an exception.

        Non-numbers or integers less than or equal to zero are unsupported.

        Args:
          self (TestBinPacker): the `TestBinpacker` instance

        Returns:
          `None`
        """
        with self.assertRaises(TypeError):
            binpacker.pack('Phillip J. Fry')

        with self.assertRaises(ValueError):
            binpacker.pack(4, 0)

        with self.assertRaises(ValueError):
            binpacker.pack(4, -1)


if __name__ == '__main__':
    unittest.main()



# Copyright 2017 Square, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pylink.jlock as jlock

import mock

import errno
import os
import unittest


class TestJLock(unittest.TestCase):
    """Tests the ``jlock`` submodule."""

    def setUp(self):
        """Called before each test.

        Performs setup.

        Args:
          self (TestJLock): the ``TestJLock`` instance

        Returns:
          ``None``
        """
        assertRaisesRegexp = getattr(self, 'assertRaisesRegexp', None)
        self.assertRaisesRegexp = getattr(self, 'assertRaisesRegex', assertRaisesRegexp)

    def tearDown(self):
        """Called after each test.

        Performs teardown.

        Args:
          self (TestJLock): the ``TestJLock`` instance

        Returns:
          ``None``
        """
        pass

    @mock.patch('tempfile.tempdir', new='tmp')
    def test_jlock_init_and_delete(self):
        """Tests initialization and deleting a ``JLock``.

        Args:
          self (TestJLock): the ``TestJLock`` instance

        Returns:
          ``None``
        """
        serial_no = 0xdeadbeef

        lock = jlock.JLock(serial_no)
        lock.release = mock.Mock()

        del lock

    @mock.patch('tempfile.tempdir', new='tmp')
    @mock.patch('os.close')
    @mock.patch('os.path.exists')
    @mock.patch('os.open')
    @mock.patch('os.write')
    @mock.patch('os.remove')
    @mock.patch('pylink.jlock.psutil')
    @mock.patch('pylink.jlock.open')
    def test_jlock_acquire_exists(self, mock_open, mock_util, mock_rm, mock_wr, mock_op, mock_exists, mock_close):
        """Tests trying to acquire when the lock exists for an active process.

        Args:
          self (TestJLock): the ``TestJLock`` instance
          mock_open (Mock): mocked built-in open method
          mock_util (Mock): mocked ``psutil`` module
          mock_rm (Mock): mocked os remove method
          mock_wr (Mock): mocked os write method
          mock_op (Mock): mocked os open method
          mock_exists (Mock): mocked path exist method
          mock_close (Mock): mocked os file close method

        Returns:
          ``None``
        """
        pid = 42
        serial_no = 0xdeadbeef

        mock_open.side_effect = [
            mock.mock_open(read_data='%s\n' % pid).return_value,
        ]

        mock_exists.side_effect = [True, True]
        mock_util.pid_exists.return_value = True
        mock_op.side_effect = [OSError(errno.EEXIST, '')]

        lock = jlock.JLock(serial_no)
        lock.release = mock.Mock()

        self.assertFalse(lock.acquired)
        self.assertFalse(lock.acquire())
        self.assertFalse(lock.acquired)

        mock_open.assert_called_once()
        mock_util.pid_exists.assert_called_with(pid)
        mock_op.assert_called_once()
        mock_rm.assert_not_called()
        mock_wr.assert_not_called()

    @mock.patch('tempfile.tempdir', new='tmp')
    @mock.patch('os.close')
    @mock.patch('os.path.exists')
    @mock.patch('os.open')
    @mock.patch('os.write')
    @mock.patch('os.remove')
    @mock.patch('pylink.jlock.psutil')
    @mock.patch('pylink.jlock.open')
    def test_jlock_acquire_os_error(self, mock_open, mock_util, mock_rm, mock_wr, mock_op, mock_exists, mock_close):
        """Tests trying to acquire the lock but generating an os-level error.

        Args:
          self (TestJLock): the ``TestJLock`` instance
          mock_open (Mock): mocked built-in open method
          mock_util (Mock): mocked ``psutil`` module
          mock_rm (Mock): mocked os remove method
          mock_wr (Mock): mocked os write method
          mock_op (Mock): mocked os open method
          mock_exists (Mock): mocked path exist method
          mock_close (Mock): mocked os file close method

        Returns:
          ``None``
        """
        serial_no = 0xdeadbeef

        mock_exists.side_effect = [False, False]
        mock_op.side_effect = [OSError(~errno.EEXIST, 'Message')]

        lock = jlock.JLock(serial_no)
        lock.release = mock.Mock()

        self.assertFalse(lock.acquired)

        with self.assertRaisesRegexp(OSError, 'Message'):
            lock.acquire()

        self.assertFalse(lock.acquired)

        mock_open.assert_not_called()
        mock_util.pid_exists.assert_not_called()
        mock_op.assert_called_once()
        mock_rm.assert_not_called()
        mock_wr.assert_not_called()

    @mock.patch('tempfile.tempdir', new='tmp')
    @mock.patch('os.close')
    @mock.patch('os.path.exists')
    @mock.patch('os.open')
    @mock.patch('os.write')
    @mock.patch('os.remove')
    @mock.patch('pylink.jlock.psutil')
    @mock.patch('pylink.jlock.open')
    def test_jlock_acquire_bad_file(self, mock_open, mock_util, mock_rm, mock_wr, mock_op, mock_exists, mock_close):
        """Tests acquiring the lockfile when the current lockfile is invallid.

        Args:
          self (TestJLock): the ``TestJLock`` instance
          mock_open (Mock): mocked built-in open method
          mock_util (Mock): mocked ``psutil`` module
          mock_rm (Mock): mocked os remove method
          mock_wr (Mock): mocked os write method
          mock_op (Mock): mocked os open method
          mock_exists (Mock): mocked path exist method
          mock_close (Mock): mocked os file close method

        Returns:
          ``None``
        """
        pid = 42
        fd = 1
        serial_no = 0xdeadbeef

        mock_open.side_effect = [
            IOError()
        ]

        mock_exists.return_value = True
        mock_op.return_value = fd

        lock = jlock.JLock(serial_no)
        lock.release = mock.Mock()

        self.assertFalse(lock.acquired)
        self.assertTrue(lock.acquire())
        self.assertTrue(lock.acquired)

        mock_exists.assert_called_once()
        mock_open.assert_called_once()
        mock_util.pid_exists.assert_not_called()
        mock_rm.assert_not_called()
        mock_op.assert_called_once()
        mock_wr.assert_called_once()

    @mock.patch('tempfile.tempdir', new='tmp')
    @mock.patch('os.close')
    @mock.patch('os.path.exists')
    @mock.patch('os.open')
    @mock.patch('os.write')
    @mock.patch('os.remove')
    @mock.patch('pylink.jlock.psutil')
    @mock.patch('pylink.jlock.open')
    def test_jlock_acquire_invalid_pid(self, mock_open, mock_util, mock_rm, mock_wr, mock_op, mock_exists, mock_close):
        """Tests acquiring the lockfile when the pid in the lockfile is invalid.

        Args:
          self (TestJLock): the ``TestJLock`` instance
          mock_open (Mock): mocked built-in open method
          mock_util (Mock): mocked ``psutil`` module
          mock_rm (Mock): mocked os remove method
          mock_wr (Mock): mocked os write method
          mock_op (Mock): mocked os open method
          mock_exists (Mock): mocked path exist method
          mock_close (Mock): mocked os file close method

        Returns:
          ``None``
        """
        fd = 1
        serial_no = 0xdeadbeef

        mock_open.side_effect = [
            mock.mock_open(read_data='dog\n').return_value,
        ]

        mock_op.return_value = fd

        lock = jlock.JLock(serial_no)
        lock.release = mock.Mock()

        self.assertFalse(lock.acquired)
        self.assertTrue(lock.acquire())
        self.assertTrue(lock.acquired)

        mock_exists.assert_called_once()
        mock_open.assert_called_once()
        mock_util.pid_exists.assert_not_called()
        mock_rm.assert_called_once()
        mock_op.assert_called_once()
        mock_wr.assert_called_once()

    @mock.patch('tempfile.tempdir', new='tmp')
    @mock.patch('os.close')
    @mock.patch('os.path.exists')
    @mock.patch('os.open')
    @mock.patch('os.write')
    @mock.patch('os.remove')
    @mock.patch('pylink.jlock.psutil')
    @mock.patch('pylink.jlock.open')
    def test_jlock_acquire_old_pid(self, mock_open, mock_util, mock_rm, mock_wr, mock_op, mock_exists, mock_close):
        """Tests acquiring when the PID in the lockfile does not exist.

        Args:
          self (TestJLock): the ``TestJLock`` instance
          mock_open (Mock): mocked built-in open method
          mock_util (Mock): mocked ``psutil`` module
          mock_rm (Mock): mocked os remove method
          mock_wr (Mock): mocked os write method
          mock_op (Mock): mocked os open method
          mock_exists (Mock): mocked path exist method
          mock_close (Mock): mocked os file close method

        Returns:
          ``None``
        """
        fd = 1
        serial_no = 0xdeadbeef

        mock_open.side_effect = [
            mock.mock_open(read_data='42\n').return_value,
        ]

        mock_op.return_value = fd
        mock_util.pid_exists.return_value = False

        lock = jlock.JLock(serial_no)
        lock.release = mock.Mock()

        self.assertFalse(lock.acquired)
        self.assertTrue(lock.acquire())
        self.assertTrue(lock.acquired)

        mock_exists.assert_called_once()
        mock_open.assert_called_once()
        mock_util.pid_exists.assert_called_once_with(42)
        mock_rm.assert_called()
        mock_op.assert_called_once()
        mock_wr.assert_called_once()

    @mock.patch('tempfile.tempdir', new='tmp')
    @mock.patch('os.path.exists')
    @mock.patch('os.close')
    @mock.patch('os.remove')
    def test_jlock_release_acquired(self, mock_remove, mock_close, mock_exists):
        """Tests releasing a held lock.

        Args:
          self (TestJLock): the ``TestJLock`` instance
          mock_remove (Mock): mock file removal method
          mock_close (Mock): mocked close method
          mock_exists (Mock): mocked path exist method

        Returns:
          ``None``
        """
        lock = jlock.JLock(0xdeadbeef)
        lock.acquired = True
        lock.fd = 1
        lock.path = os.sep

        self.assertTrue(lock.release())

        mock_exists.return_value = True
        mock_remove.assert_called_once_with(os.sep)
        mock_close.assert_called_once_with(1)
        mock_exists.assert_called_once_with(os.sep)

        self.assertEqual(False, lock.acquired)

    @mock.patch('tempfile.tempdir', new='tmp')
    def test_jlock_release_not_held(self):
        """Tests calling release when lock not held.

        Args:
          self (TestJLock): the ``TestJLock`` instance

        Returns:
          ``None``
        """
        lock = jlock.JLock(0xdeadbeef)
        self.assertFalse(lock.release())


if __name__ == '__main__':
    unittest.main()



# Copyright 2017 Square, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pylink
import pylink.__main__ as main

import logging
import mock

try:
    import StringIO
except ImportError:
    import io as StringIO
import ctypes
import sys
import unittest


class TestMain(unittest.TestCase):
    """Tests the command-line interface."""

    def setUp(self):
        """Called before each test.

        Performs setup.

        Args:
          self (TestMain): the ``TestMain`` instance

        Returns:
          ``None``
        """
        pass

    def tearDown(self):
        """Called after each test.

        Performs teardown.

        Args:
          self (TestMain): the ``TestMain`` instance

        Returns:
          ``None``
        """
        pass

    @mock.patch('sys.stderr', new_callable=StringIO.StringIO)
    @mock.patch('sys.stdout', new_callable=StringIO.StringIO)
    @mock.patch('pylink.__main__.pylink.JLink')
    def test_help(self, mock_jlink, mock_stdout, mock_stderr):
        """Tests printing out the command-line help.

        Args:
          self (TestMain): the ``TestMain`` instance
          mock_jlink (mock.Mock): the mocked ``JLink`` object
          mock_stdout (mock.Mock): mocked standard output stream
          mock_stderr (mock.Mock): mocked standard error stream

        Returns:
          ``None``
        """
        with self.assertRaises(SystemExit):
            main.main(['--help'])

        self.assertTrue('usage: pylink' in mock_stdout.getvalue())

    @mock.patch('sys.argv', ['pylink', '--help'])
    @mock.patch('sys.stderr', new_callable=StringIO.StringIO)
    @mock.patch('sys.stdout', new_callable=StringIO.StringIO)
    @mock.patch('pylink.__main__.pylink.JLink')
    def test_help_argv(self, mock_jlink, mock_stdout, mock_stderr):
        """Tests printing out the command-line help when called without args.

        Args:
          self (TestMain): the ``TestMain`` instance
          mock_jlink (mock.Mock): the mocked ``JLink`` object
          mock_stdout (mock.Mock): mocked standard output stream
          mock_stderr (mock.Mock): mocked standard error stream

        Returns:
          ``None``
        """
        with self.assertRaises(SystemExit):
            main.main()

        self.assertTrue('usage: pylink' in mock_stdout.getvalue())

    @mock.patch('sys.stderr', new_callable=StringIO.StringIO)
    @mock.patch('sys.stdout', new_callable=StringIO.StringIO)
    @mock.patch('pylink.__main__.pylink.JLink')
    def test_version_command(self, mock_jlink, mock_stdout, mock_stderr):
        """Tests printing the version of the module.

        Args:
          self (TestMain): the ``TestMain`` instance
          mock_jlink (mock.Mock): the mocked ``JLink`` object
          mock_stdout (mock.Mock): mocked standard output stream
          mock_stderr (mock.Mock): mocked standard error stream

        Returns:
          ``None``
        """
        with self.assertRaises(SystemExit):
            main.main(['--version'])

        expected = 'pylink %s' % pylink.__version__
        if sys.version_info >= (3, 0):
            self.assertEqual(expected, mock_stdout.getvalue().strip())
        else:
            self.assertEqual(expected, mock_stderr.getvalue().strip())

    @mock.patch('pylink.__main__.logging.basicConfig')
    @mock.patch('sys.stderr', new_callable=StringIO.StringIO)
    @mock.patch('sys.stdout', new_callable=StringIO.StringIO)
    @mock.patch('pylink.__main__.pylink.JLink')
    def test_verbosity(self, mock_jlink, mock_stdout, mock_stderr, mock_config):
        """Tests setting the verbosity of the command-line tool.

        Args:
          self (TestMain): the ``TestMain`` instance
          mock_jlink (mock.Mock): the mocked ``JLink`` object
          mock_stdout (mock.Mock): mocked standard output stream
          mock_stderr (mock.Mock): mocked standard error stream
          mock_config (mock.Mock): mocked logging configuration function

        Returns:
          ``None``
        """
        args = ['emulator', '--test']

        # No levels of verbosity.
        self.assertEqual(0, main.main(args))
        mock_config.assert_called_with(level=logging.WARNING)

        # One level of verbosity.
        self.assertEqual(0, main.main(['-v'] + args))
        mock_config.assert_called_with(level=logging.INFO)

        # Two levels of verbosity.
        self.assertEqual(0, main.main(['-v', '-v'] + args))
        mock_config.assert_called_with(level=logging.DEBUG)

        # Three levels of verbosity.
        self.assertEqual(0, main.main(['-v', '-v', '-v'] + args))
        mock_config.assert_called_with(level=logging.DEBUG)

    @mock.patch('sys.stderr', new_callable=StringIO.StringIO)
    @mock.patch('sys.stdout', new_callable=StringIO.StringIO)
    @mock.patch('pylink.__main__.pylink.JLink')
    def test_new_command_fail_validation(self, mock_jlink, mock_stdout, mock_stderr):
        """Tests creating a new command that fails validation.

        Args:
          self (TestMain): the ``TestMain`` instance
          mock_jlink (mock.Mock): the mocked ``JLink`` object
          mock_stdout (mock.Mock): mocked standard output stream
          mock_stderr (mock.Mock): mocked standard error stream

        Returns:
          ``None``
        """
        with self.assertRaises(ValueError):
            class a(main.Command):
                description = 'description'
                help = 'help'

        with self.assertRaises(ValueError):
            class a(main.Command):
                name = 'name'
                help = 'help'

        with self.assertRaises(ValueError):
            class a(main.Command):
                name = 'name'
                description = 'description'

    @mock.patch('sys.stderr', new_callable=StringIO.StringIO)
    @mock.patch('sys.stdout', new_callable=StringIO.StringIO)
    @mock.patch('pylink.__main__.pylink.JLink')
    def test_new_command_method_not_implemented(self, mock_jlink, mock_stdout, mock_stderr):
        """Tests when a command is created, errors if method not implemeneted.

        Args:
          self (TestMain): the ``TestMain`` instance
          mock_jlink (mock.Mock): the mocked ``JLink`` object
          mock_stdout (mock.Mock): mocked standard output stream
          mock_stderr (mock.Mock): mocked standard error stream

        Returns:
          ``None``
        """
        class A(main.Command):
            name = 'name'
            description = 'description'
            help = 'help'

        with self.assertRaises(NotImplementedError):
            a = A()
            a.add_arguments(None)

        with self.assertRaises(NotImplementedError):
            a = A()
            a.run(None)

        main.CommandMeta.registry.pop('A')

    @mock.patch('sys.stderr', new_callable=StringIO.StringIO)
    @mock.patch('sys.stdout', new_callable=StringIO.StringIO)
    @mock.patch('pylink.__main__.pylink.JLink')
    def test_main_jlinkexception(self, mock_jlink, mock_stdout, mock_stderr):
        """Tests when a J-Link exception is raised when a command is run.

        Args:
          self (TestMain): the ``TestMain`` instance
          mock_jlink (mock.Mock): the mocked ``JLink`` object
          mock_stdout (mock.Mock): mocked standard output stream
          mock_stderr (mock.Mock): mocked standard error stream

        Returns:
          ``None``
        """
        args = ['emulator', '--test']
        mock_jlink.side_effect = pylink.JLinkException('error')
        self.assertEqual(1, main.main(args))
        self.assertEqual('Error: error', mock_stderr.getvalue().strip())

    @mock.patch('sys.stderr', new_callable=StringIO.StringIO)
    @mock.patch('sys.stdout', new_callable=StringIO.StringIO)
    @mock.patch('pylink.__main__.pylink.JLink')
    def test_emulator_test_command(self, mock_jlink, mock_stdout, mock_stderr):
        """Tests the emulator self-test command.

        Args:
          self (TestMain): the ``TestMain`` instance
          mock_jlink (mock.Mock): the mocked ``JLink`` object
          mock_stdout (mock.Mock): mocked standard output stream
          mock_stderr (mock.Mock): mocked standard error stream

        Returns:
          ``None``
        """
        args = ['emulator', '--test']

        mocked = mock.Mock()
        mock_jlink.return_value = mocked

        mocked.test.return_value = True
        self.assertEqual(0, main.main(args))
        self.assertEqual('Self-test succeeded.', mock_stdout.getvalue().strip())

        mock_stdout.truncate(0)
        mock_stdout.seek(0)

        mocked.test.return_value = False
        self.assertEqual(0, main.main(args))
        self.assertEqual('Self-test failed.', mock_stdout.getvalue().strip())

    @mock.patch('sys.stderr', new_callable=StringIO.StringIO)
    @mock.patch('sys.stdout', new_callable=StringIO.StringIO)
    @mock.patch('pylink.__main__.pylink.JLink')
    def test_emulator_list_command(self, mock_jlink, mock_stdout, mock_stderr):
        """Tests the emulator list device command.

        Args:
          self (TestMain): the ``TestMain`` instance
          mock_jlink (mock.Mock): the mocked ``JLink`` object
          mock_stdout (mock.Mock): mocked standard output stream
          mock_stderr (mock.Mock): mocked standard error stream

        Returns:
          ``None``
        """
        usb_device = pylink.JLinkConnectInfo()
        usb_device.SerialNumber = 123456789
        usb_device.acProduct = b'J-Trace CM'
        usb_device.Connection = 1

        ip_device = pylink.JLinkConnectInfo()
        ip_device.SerialNumber = 987654321
        ip_device.acProduct = b'J-Link PRO'
        ip_device.Nickname = b'J-Link PRO'
        ip_device.acFWString = b'J-Link PRO compiled Mon. Nov. 7th 13:56:44'
        ip_device.Connection = 0

        mocked = mock.Mock()
        mock_jlink.return_value = mocked

        args = ['emulator', '--list']
        mocked.connected_emulators.return_value = [usb_device, ip_device]
        self.assertEqual(0, main.main(args))
        mocked.connected_emulators.assert_called_with(pylink.JLinkHost.USB_OR_IP)
        self.assertTrue('Product Name: J-Trace CM' in mock_stdout.getvalue())
        self.assertTrue('Product Name: J-Link PRO' in mock_stdout.getvalue())

        mock_stdout.truncate(0)
        mock_stdout.seek(0)

        args = ['emulator', '--list', 'usb']
        mocked.connected_emulators.return_value = [usb_device]
        self.assertEqual(0, main.main(args))
        mocked.connected_emulators.assert_called_with(pylink.JLinkHost.USB)
        self.assertTrue('Product Name: J-Trace CM' in mock_stdout.getvalue())
        self.assertTrue('Connection: USB' in mock_stdout.getvalue())
        self.assertFalse('Product Name: J-Link PRO' in mock_stdout.getvalue())

        mock_stdout.truncate(0)
        mock_stdout.seek(0)

        args = ['emulator', '--list', 'ip']
        mocked.connected_emulators.return_value = [ip_device]
        self.assertEqual(0, main.main(args))
        mocked.connected_emulators.assert_called_with(pylink.JLinkHost.IP)
        self.assertFalse('Product Name: J-Trace CM' in mock_stdout.getvalue())
        self.assertTrue('Connection: IP' in mock_stdout.getvalue())
        self.assertTrue('Product Name: J-Link PRO' in mock_stdout.getvalue())

    @mock.patch('sys.stderr', new_callable=StringIO.StringIO)
    @mock.patch('sys.stdout', new_callable=StringIO.StringIO)
    @mock.patch('pylink.__main__.pylink.JLink')
    def test_emulator_supported_command(self, mock_jlink, mock_stdout, mock_stderr):
        """Tests querying whether a device is supported.

        Args:
          self (TestMain): the ``TestMain`` instance
          mock_jlink (mock.Mock): the mocked ``JLink`` object
          mock_stdout (mock.Mock): mocked standard output stream
          mock_stderr (mock.Mock): mocked standard error stream

        Returns:
          ``None``
        """
        mocked = mock.Mock()
        mock_jlink.return_value = mocked

        name = (ctypes.c_char * 10)(*b'CANADA')
        device = pylink.JLinkDeviceInfo()
        device.sName = name

        mocked.get_device_index.side_effect = pylink.errors.JLinkException('Unsupported device selected.')

        args = ['emulator', '-s', 'USA']
        self.assertEqual(0, main.main(args))
        self.assertEqual('USA is not supported :(', mock_stdout.getvalue().strip())

        mock_stdout.truncate(0)
        mock_stdout.seek(0)

        mocked.get_device_index.side_effect = None
        mocked.get_device_index.return_value = 1
        mocked.supported_device.return_value = device

        args = ['emulator', '-s', 'CANADA']
        self.assertEqual(0, main.main(args))
        self.assertTrue('Device Name: CANADA' in mock_stdout.getvalue())

    @mock.patch('sys.stderr', new_callable=StringIO.StringIO)
    @mock.patch('sys.stdout', new_callable=StringIO.StringIO)
    @mock.patch('pylink.__main__.pylink.JLink')
    def test_info_product_command(self, mock_jlink, mock_stdout, mock_stderr):
        """Tests the product information command.

        Args:
          self (TestMain): the ``TestMain`` instance
          mock_jlink (mock.Mock): the mocked ``JLink`` object
          mock_stdout (mock.Mock): mocked standard output stream
          mock_stderr (mock.Mock): mocked standard error stream

        Returns:
          ``None``
        """
        mocked = mock.Mock()
        mock_jlink.return_value = mocked

        mocked.features = ['RDI', 'FlashBP']

        args = ['info', '--product', '--serial', '123456789']
        self.assertEqual(0, main.main(args))

        self.assertTrue('Product' in mock_stdout.getvalue())
        self.assertTrue('Features: RDI, FlashBP' in mock_stdout.getvalue())

    @mock.patch('sys.stderr', new_callable=StringIO.StringIO)
    @mock.patch('sys.stdout', new_callable=StringIO.StringIO)
    @mock.patch('pylink.__main__.pylink.JLink')
    def test_info_pin_command(self, mock_jlink, mock_stdout, mock_stderr):
        """Tests the JTAG pin status information command.

        Args:
          self (TestMain): the ``TestMain`` instance
          mock_jlink (mock.Mock): the mocked ``JLink`` object
          mock_stdout (mock.Mock): mocked standard output stream
          mock_stderr (mock.Mock): mocked standard error stream

        Returns:
          ``None``
        """
        mocked = mock.Mock()
        mock_jlink.return_value = mocked

        status = pylink.JLinkHardwareStatus()
        status.VTarget = 80
        status.tck = 0
        status.tdi = 0
        status.tdo = 0
        status.tms = 1
        status.tres = 0
        status.trst = 1

        mocked.hardware_status = status

        args = ['info', '--jtag', '--serial', '123456789']
        self.assertEqual(0, main.main(args))

        self.assertTrue('TCK Pin Status: 0' in mock_stdout.getvalue())
        self.assertTrue('TRST Pin Status: 1' in mock_stdout.getvalue())

    @mock.patch('sys.stderr', new_callable=StringIO.StringIO)
    @mock.patch('sys.stdout', new_callable=StringIO.StringIO)
    @mock.patch('pylink.__main__.pylink.JLink')
    def test_firmware_upgrade_command(self, mock_jlink, mock_stdout, mock_stderr):
        """Tests the command t- upgrade the J-Link firmware.

        Args:
          self (TestMain): the ``TestMain`` instance
          mock_jlink (mock.Mock): the mocked ``JLink`` object
          mock_stdout (mock.Mock): mocked standard output stream
          mock_stderr (mock.Mock): mocked standard error stream

        Returns:
          ``None``
        """
        mocked = mock.Mock()
        mock_jlink.return_value = mocked

        args = ['firmware', '--upgrade', '--serial', '123456789']

        # Firmware is already new, can't upgrade anymore.
        mocked.firmware_outdated.return_value = False
        self.assertEqual(0, main.main(args))
        self.assertTrue('DLL firmware is not newer' in mock_stdout.getvalue())
        self.assertEqual(0, mocked.update_firmware.call_count)

        mock_stdout.truncate(0)
        mock_stdout.seek(0)

        # Firmware is older, so we can upgrade.
        mocked.firmware_outdated.return_value = True
        mocked.update_firmware.side_effect = pylink.JLinkException('message')
        self.assertEqual(0, main.main(args))
        mocked.update_firmware.assert_called_once()
        self.assertTrue('Firmware Updated' in mock_stdout.getvalue())

    @mock.patch('sys.stderr', new_callable=StringIO.StringIO)
    @mock.patch('sys.stdout', new_callable=StringIO.StringIO)
    @mock.patch('pylink.__main__.pylink.JLink')
    def test_firmware_downgrade_command(self, mock_jlink, mock_stdout, mock_stderr):
        """Tests the command to downgrade the J-Link firmware.

        Args:
          self (TestMain): the ``TestMain`` instance
          mock_jlink (mock.Mock): the mocked ``JLink`` object
          mock_stdout (mock.Mock): mocked standard output stream
          mock_stderr (mock.Mock): mocked standard error stream

        Returns:
          ``None``
        """
        mocked = mock.Mock()
        mock_jlink.return_value = mocked

        args = ['firmware', '--downgrade', '--serial', '123456789']

        # Firmware is older than the DLL firmware.
        mocked.firmware_newer.return_value = False
        self.assertEqual(0, main.main(args))
        self.assertTrue('DLL firmware is not older' in mock_stdout.getvalue())
        self.assertEqual(0, mocked.update_firmware.call_count)

        mock_stdout.truncate(0)
        mock_stdout.seek(0)

        # Firmware is newer, so we can downgrade.
        mocked.firmware_newer.return_value = True
        mocked.update_firmware.side_effect = pylink.JLinkException('message')
        self.assertEqual(0, main.main(args))
        mocked.invalidate_firmware.assert_called_once()
        mocked.update_firmware.assert_called_once()
        self.assertTrue('Firmware Downgraded' in mock_stdout.getvalue())

    @mock.patch('sys.stderr', new_callable=StringIO.StringIO)
    @mock.patch('sys.stdout', new_callable=StringIO.StringIO)
    @mock.patch('pylink.__main__.pylink.JLink')
    def test_flash_command(self, mock_jlink, mock_stdout, mock_stderr):
        """Tests running the flash command over JTAG and SWD.

        Args:
          self (TestMain): the ``TestMain`` instance
          mock_jlink (mock.Mock): the mocked ``JLink`` object
          mock_stdout (mock.Mock): mocked standard output stream
          mock_stderr (mock.Mock): mocked standard error stream

        Returns:
          ``None``
        """
        mocked = mock.Mock()
        mock_jlink.return_value = mocked

        args = ['flash', '-t', 'swd', '-d', 'DEVICE', '-s', '123456789', 'fileA']
        self.assertEqual(0, main.main(args))
        mocked.flash_file.assert_called_with(addr=0,
                                             on_progress=pylink.util.flash_progress_callback,
                                             path='fileA')

        args = ['flash', '-t', 'jtag', '-d', 'DEVICE',  '-s', '123456789', 'fileB']
        self.assertEqual(0, main.main(args))
        mocked.flash_file.assert_called_with(addr=0,
                                             on_progress=pylink.util.flash_progress_callback,
                                             path='fileB')

    @mock.patch('pylink.unlock')
    @mock.patch('sys.stderr', new_callable=StringIO.StringIO)
    @mock.patch('sys.stdout', new_callable=StringIO.StringIO)
    @mock.patch('pylink.__main__.pylink.JLink')
    def test_unlock_command(self, mock_jlink, mock_stdout, mock_stderr, mock_unlock):
        """Tests the command for unlocking a locked device.

        Args:
          self (TestMain): the ``TestMain`` instance
          mock_jlink (mock.Mock): the mocked ``JLink`` object
          mock_stdout (mock.Mock): mocked standard output stream
          mock_stderr (mock.Mock): mocked standard error stream
          mock_unlock (mock.Mock): mocked unlock device call

        Returns:
          ``None``
        """
        mocked = mock.Mock()
        mock_jlink.return_value = mocked

        args = ['unlock', '-t', 'swd', '-d', 'DEVICE', '-s', '123456789', 'kinetis']

        mock_unlock.return_value = True
        self.assertEqual(0, main.main(args))
        self.assertEqual('Successfully unlocked device!', mock_stdout.getvalue().strip())

        mock_stdout.truncate(0)
        mock_stdout.seek(0)

        mock_unlock.return_value = False
        self.assertEqual(0, main.main(args))
        self.assertEqual('Failed to unlock device!', mock_stdout.getvalue().strip())

    @mock.patch('sys.stderr', new_callable=StringIO.StringIO)
    @mock.patch('sys.stdout', new_callable=StringIO.StringIO)
    @mock.patch('pylink.__main__.pylink.JLink')
    def test_erase_command(self, mock_jlink, mock_stdout, mock_stderr):
        """Tests the command for erasing the device.

        Args:
          self (TestMain): the ``TestMain`` instance
          mock_jlink (mock.Mock): the mocked ``JLink`` object
          mock_stdout (mock.Mock): mocked standard output stream
          mock_stderr (mock.Mock): mocked standard error stream

        Returns:
          ``None``
        """
        mocked = mock.Mock()
        mock_jlink.return_value = mocked
        mocked.erase.return_value = 1337

        args = ['erase', '-t', 'swd', '-d', 'DEVICE', '-s', '123456789']

        self.assertEqual(0, main.main(args))
        self.assertEqual('Bytes Erased: 1337', mock_stdout.getvalue().strip())

    @mock.patch('sys.stderr', new_callable=StringIO.StringIO)
    @mock.patch('sys.stdout', new_callable=StringIO.StringIO)
    @mock.patch('pylink.__main__.pylink.JLink')
    def test_license_list_command(self, mock_jlink, mock_stdout, mock_stderr):
        """Tests the command for listing emulator licenses.

        Args:
          self (TestMain): the ``TestMain`` instance
          mock_jlink (mock.Mock): the mocked ``JLink`` object
          mock_stdout (mock.Mock): mocked standard output stream
          mock_stderr (mock.Mock): mocked standard error stream

        Returns:
          ``None``
        """
        mocked = mock.Mock()
        mock_jlink.return_value = mocked

        mocked.licenses = 'FlashBP,RDI'
        mocked.custom_licenses = 'GDB'

        args = ['license', '-l', '-s', '123456789']
        self.assertEqual(0, main.main(args))
        self.assertTrue('Built-in Licenses: FlashBP, RDI' in mock_stdout.getvalue())
        self.assertTrue('Custom Licenses: GDB' in mock_stdout.getvalue())

    @mock.patch('sys.stderr', new_callable=StringIO.StringIO)
    @mock.patch('sys.stdout', new_callable=StringIO.StringIO)
    @mock.patch('pylink.__main__.pylink.JLink')
    def test_license_add_command(self, mock_jlink, mock_stdout, mock_stderr):
        """Tests the command for adding emulator licenses.

        Args:
          self (TestMain): the ``TestMain`` instance
          mock_jlink (mock.Mock): the mocked ``JLink`` object
          mock_stdout (mock.Mock): mocked standard output stream
          mock_stderr (mock.Mock): mocked standard error stream

        Returns:
          ``None``
        """
        mocked = mock.Mock()
        mock_jlink.return_value = mocked

        args = ['license', '-a', 'GDB', '-s', '123456789']

        mocked.add_license.return_value = True
        self.assertEqual(0, main.main(args))
        self.assertEqual('Successfully added license.', mock_stdout.getvalue().strip())

        mock_stdout.truncate(0)
        mock_stdout.seek(0)

        mocked.add_license.return_value = False
        self.assertEqual(0, main.main(args))
        self.assertEqual('License already exists.', mock_stdout.getvalue().strip())

    @mock.patch('sys.stderr', new_callable=StringIO.StringIO)
    @mock.patch('sys.stdout', new_callable=StringIO.StringIO)
    @mock.patch('pylink.__main__.pylink.JLink')
    def test_license_erase_command(self, mock_jlink, mock_stdout, mock_stderr):
        """Tests the command for erasing licenses.

        Args:
          self (TestMain): the ``TestMain`` instance
          mock_jlink (mock.Mock): the mocked ``JLink`` object
          mock_stdout (mock.Mock): mocked standard output stream
          mock_stderr (mock.Mock): mocked standard error stream

        Returns:
          ``None``
        """
        mocked = mock.Mock()
        mock_jlink.return_value = mocked

        args = ['license', '-e', '-s', '123456789']

        mocked.erase_licenses.return_value = True
        self.assertEqual(0, main.main(args))
        self.assertEqual('Successfully erased all custom licenses.', mock_stdout.getvalue().strip())

        mock_stdout.truncate(0)
        mock_stdout.seek(0)

        mocked.erase_licenses.return_value = False
        self.assertEqual(0, main.main(args))
        self.assertEqual('Failed to erase custom licenses.', mock_stdout.getvalue().strip())


if __name__ == '__main__':
    unittest.main()



# Copyright 2017 Square, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import behave

import time


@behave.when('I enable semihosting')
def step_enable_semihosting(context):
    """Enables semihosting by setting a SVC break instruction.

    Args:
      context (Context): the ``Context`` instance

    Returns:
      ``None``
    """
    jlink = context.jlink
    jlink.set_vector_catch(1 << 2)


@behave.then('I should see over SWD')
def step_should_see_swd(context):
    """Asserts that the text over the SWD interface matches the given text.

    Args:
      context (Context): the ``Context`` instance

    Returns:
      ``None``
    """
    jlink = context.jlink
    expected_strings = context.text.split('\n')

    # r0 holds the type of command that resulted in the SVC exception.  For a
    # full list of what these commands can be, please see the J-Link
    # documentation on semihosting ``UM8001_JLink.pdf``
    command_register = 0x0

    # r1 holds a pointer to an address on the target that holds the additional
    # function arguments as 32-bit aligned words.
    command_info_register = 0x1

    while len(expected_strings) > 0:
        if not jlink.halted():
            time.sleep(1)
            continue

        # The CPU is halted after a Vector Catch, so we can now examine why
        # the CPU halted.
        sys_write = 0x05
        command = jlink.register_read(command_register)
        assert command == sys_write

        command_info = jlink.register_read(command_info_register)
        handle, ptr, num_bytes = jlink.memory_read32(command_info, 3)
        actual_string = ''.join(map(chr, jlink.memory_read8(ptr, num_bytes))).strip()
        print(actual_string)

        actual_stripped = ''.join(actual_string.strip().split())
        expected_stripped = ''.join(expected_strings.pop(0).strip().split())
        assert actual_stripped.startswith(expected_stripped)

        # Resume execution to the original mode.  Have to write the result of
        # the SVC call (always 0), and step over the breakpointed instruction.
        jlink.register_write(0x0, 0)
        jlink.step(thumb=True)
        assert jlink.restart(num_instructions=2, skip_breakpoints=True)



# Copyright 2017 Square, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys

# Stub the 'psutil' module that is required by this package in order to enable
# the main module to be imported.
try:
    import psutil
except ImportError:
    sys.modules['psutil'] = {}

try:
    import six
except ImportError:
    sys.modules['six'] = {}

import pylink

import os
import setuptools
import shutil
import subprocess


class CleanCommand(setuptools.Command):
    """Custom clean command."""

    description = 'clean the project build files'

    user_options = []

    def initialize_options(self):
        """Initialize the Command.

        Args:
          self (CleanCommand): the ``CleanCommand`` instance

        Returns:
          ``None``
        """
        self.cwd = None
        self.build_dirs = None
        self.build_artifacts = None

    def finalize_options(self):
        """Populate the attributes.

        Args:
          self (CleanCommand): the ``CleanCommand`` instance

        Returns:
          ``None``
        """
        self.cwd = os.path.abspath(os.path.dirname(__file__))
        self.build_dirs = [
            os.path.join(self.cwd, 'build'),
            os.path.join(self.cwd, 'htmlcov'),
            os.path.join(self.cwd, 'dist'),
            os.path.join(self.cwd, 'pylink_square.egg-info')
        ]
        self.build_artifacts = ['.pyc', '.o', '.elf', '.bin']

    def run(self):
        """Runs the command.

        Args:
          self (CleanCommand): the ``CleanCommand`` instance

        Returns:
          ``None``
        """
        for build_dir in self.build_dirs:
            if os.path.isdir(build_dir):
                sys.stdout.write('Removing %s%s' % (build_dir, os.linesep))
                shutil.rmtree(build_dir)

        for (root, dirs, files) in os.walk(self.cwd):
            for name in files:
                fullpath = os.path.join(root, name)
                if any(fullpath.endswith(ext) for ext in self.build_artifacts):
                    sys.stdout.write('Removing %s%s' % (fullpath, os.linesep))
                    os.remove(fullpath)


class CoverageCommand(setuptools.Command):
    """Custom command for generating coverage information."""

    description = 'generate coverage information'

    user_options = []

    def initialize_options(self):
        """Initializes the command.

        Args:
          self (CoverageCommand): the ``CoverageCommand`` instance

        Returns:
          ``None``
        """
        self.cwd = None
        self.test_dir = None

    def finalize_options(self):
        """Finalizes the command's options.

        Args:
          self (CoverageCommand): the ``CoverageCommand`` instance

        Returns:
          ``None``
        """
        self.cwd = os.path.abspath(os.path.dirname(__file__))
        self.test_dir = os.path.join(self.cwd, 'tests')

    def run(self):
        """Runs the command to generate coverage information.

        Args:
          self (CoverageCommand): the ``CoverageCommand`` instance

        Returns:
          ``None``
        """
        import coverage
        subprocess.call(['coverage', 'run', '--source', 'pylink', 'setup.py', 'test'])
        subprocess.call(['coverage', 'report'])
        subprocess.call(['coverage', 'html'])


class BDDTestCommand(setuptools.Command):
    """BDD test command."""

    description = 'run the behaviour tests'

    user_options = []

    def initialize_options(self):
        """Initialize the Command.

        Args:
          self (BDDTestCommand): the ``BDDTestCommand`` instance

        Returns:
          ``None``
        """
        self.cwd = None
        self.firmware_dirs = None
        self.features_dir = None

    def finalize_options(self):
        """Populate the attributes.

        Args:
          self (BDDTestCommand): the ``BDDTestCommand`` instance

        Returns:
          ``None``
        """
        self.cwd = os.path.abspath(os.path.dirname(__file__))
        self.features_dir = os.path.join(self.cwd, 'tests', 'functional', 'features')
        self.firmware_dirs = []

        root = os.path.join(self.cwd, 'tests', 'functional', 'firmware')
        for f in os.listdir(root):
            fullpath = os.path.join(root, f)
            if os.path.isdir(fullpath):
                self.firmware_dirs.append(fullpath)

    def run(self):
        """Runs the command.

        Args:
          self (BDDTestCommand): the ``BDDTestCommand`` instance

        Returns:
          ``True`` on success, otherwise ``False``.

        Raises:
          ValueError: if a build fails
        """
        import behave.__main__ as behave

        for d in self.firmware_dirs:
            original_dir = os.getcwd()
            os.chdir(d)

            output = ''
            try:
                output = subprocess.check_output('make', shell=True, stderr=subprocess.STDOUT)
            except subprocess.CalledProcessError as e:
                if output:
                    sys.stdout.write('Captured Output:%s%s%s' % (os.linesep, output, os.linesep))
                os.chdir(original_dir)
                raise e

            os.chdir(original_dir)

        return behave.main([self.features_dir])


def long_description():
    """Reads and returns the contents of the README.

    On failure, returns the project long description.

    Returns:
      The project's long description.
    """
    cwd = os.path.abspath(os.path.dirname(__file__))
    readme_path = os.path.join(cwd, 'README.md')
    if not os.path.exists(readme_path):
        return pylink.__long_description__

    try:
        import pypandoc
        return pypandoc.convert_file(readme_path, 'rst')
    except (IOError, ImportError):
        pass

    return open(readme_path, 'r').read()


setuptools.setup(
    # Project information.
    name='pylink-square',
    version=pylink.__version__,

    #  Metadata for upload to PyPI.
    author=pylink.__author__,
    author_email=pylink.__author_email__,
    description=pylink.__description__,
    long_description=long_description(),
    license=pylink.__license__,
    keywords='SEGGER J-Link',
    url=pylink.__url__,

    # Packages and package data.
    packages=[
        'pylink',
        'pylink.protocols',
        'pylink.unlockers'
    ],
    package_data={
        'pylink': [
        ]
    },

    # Dependencies.
    install_requires=[
        'psutil >= 5.2.2',
        'six'
    ],

    # Tests
    test_suite='tests',

    # Test requirements
    tests_require=[
        'mock == 2.0.0'
    ],

    # Additional scripts.
    scripts=[
        os.path.join('examples', 'pylink-rtt'),
        os.path.join('examples', 'pylink-swv'),
    ],

    # Entry points.
    entry_points={
        'console_scripts': [
            'pylink = pylink.__main__:main'
        ]
    },

    # Custom commands
    cmdclass={
        'clean': CleanCommand,
        'coverage': CoverageCommand,
        'bddtest': BDDTestCommand
    },

    python_requires='>=3.0',
)



# Copyright 2017 Square, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import ctypes


class IDCodeRegisterBits(ctypes.LittleEndianStructure):
    """This class holds the different bit masks for the IDCode register.

    Attributes:
      valid: validity bit, should always be ``0``.
      manufactuer: the JEDEC Manufacturer ID.
      part_no: the part number defined by the manufacturer.
      version_code: the version code.
    """
    _fields_ = [
        ('valid',        ctypes.c_uint32, 1),
        ('manufacturer', ctypes.c_uint32, 11),
        ('part_no',      ctypes.c_uint32, 16),
        ('version_code', ctypes.c_uint32, 4)
    ]


class IDCodeRegisterFlags(ctypes.Union):
    """Mask for the IDCode register bits.

    Attributes:
      value: the value stored in the mask.
    """
    _anonymous_ = ('bit',)
    _fields_ = [
        ('bit',   IDCodeRegisterBits),
        ('value', ctypes.c_uint32)
    ]


class AbortRegisterBits(ctypes.LittleEndianStructure):
    """This class holds the different bit mask for the Abort Register.

    Attributes:
      DAPABORT: write ``1`` to trigger a DAP abort.
      STKCMPCLR: write ``1`` to clear the ``STICKYCMP`` sticky compare flag
          (only supported on SW-DP).
      STKERRCLR: write ``1`` to clear the ``STICKYERR`` sticky error flag
          (only supported on SW-DP).
      WDERRCLR: write ``1`` to clear the ``WDATAERR`` write data error flag
          (only supported on SW-DP).
      ORUNERRCLR: write ``1`` to clear the ``STICKYORUN`` overrun error flag
          (only supported on SW-DP).
    """
    _fields_ = [
        ('DAPABORT',   ctypes.c_uint32, 1),
        ('STKCMPCLR',  ctypes.c_uint32, 1),
        ('STKERRCLR',  ctypes.c_uint32, 1),
        ('WDERRCLR',   ctypes.c_uint32, 1),
        ('ORUNERRCLR', ctypes.c_uint32, 1),
        ('RESERVED',   ctypes.c_uint32, 27),
    ]


class AbortRegisterFlags(ctypes.Union):
    """Mask for the abort register bits.

    Attributes:
      value: the value stored in the mask.
    """
    _anonymous_ = ('bit',)
    _fields_ = [
        ('bit',   AbortRegisterBits),
        ('value', ctypes.c_uint32)
    ]


class ControlStatusRegisterBits(ctypes.LittleEndianStructure):
    """This class holds the different bit masks for the DP Control / Status
    Register bit assignments.

    Attributes:
      ORUNDETECT: if set, enables overrun detection.
      STICKYORUN: if overrun is enabled, is set when overrun occurs.
      TRNMODE: transfer mode for acess port operations.
      STICKYCMP: is set when a match occurs on a pushed compare or verify
          operation.
      STICKYERR: is set when an error is returned by an access port
          transaction.
      READOK: is set when the response to a previous access port or ``RDBUFF``
          was ``OK``.
      WDATAERR: set to ``1`` if a Write Data Error occurs.
      MASKLANE: bytes to be masked in pushed compare and verify operations.
      TRNCNT: transaction counter.
      RESERVED: reserved.
      CDBGRSTREQ: debug reset request.
      CDBGRSTACK: debug reset acknowledge.
      CDBGPWRUPREQ: debug power-up request.
      CDBGPWRUPACK: debug power-up acknowledge.
      CSYSPWRUPREQ: system power-up request
      CSYSPWRUPACK: system power-up acknowledge.

    See also:
      See the ARM documentation on the significance of these masks
      `here <http://infocenter.arm.com/help/index.jsp?topic=/com.arm.doc.ihi0031c/index.html>`_.
    """
    _fields_ = [
        ('ORUNDETECT',   ctypes.c_uint32, 1),   # read/write
        ('STICKYORUN',   ctypes.c_uint32, 1),   # read-only
        ('TRNMODE',      ctypes.c_uint32, 2),   # read/write
        ('STICKYCMP',    ctypes.c_uint32, 1),   # read-only
        ('STICKYERR',    ctypes.c_uint32, 1),   # read-only
        ('READOK',       ctypes.c_uint32, 1),   # read-only
        ('WDATAERR',     ctypes.c_uint32, 1),   # read-only
        ('MASKLANE',     ctypes.c_uint32, 4),   # read/write
        ('TRNCNT',       ctypes.c_uint32, 12),  # read/write
        ('RESERVED',     ctypes.c_uint32, 2),   # -
        ('CDBGRSTREQ',   ctypes.c_uint32, 1),   # read/write
        ('CDBGRSTACK',   ctypes.c_uint32, 1),   # read-only
        ('CDBGPWRUPREQ', ctypes.c_uint32, 1),   # read/write
        ('CDBGPWRUPACK', ctypes.c_uint32, 1),   # read-only
        ('CSYSPWRUPREQ', ctypes.c_uint32, 1),   # read/write
        ('CSYSPWRUPACK', ctypes.c_uint32, 1)    # read-only
    ]


class ControlStatusRegisterFlags(ctypes.Union):
    """Mask for the control/status register bits.

    Attributes:
      value: the value stored in the mask.
    """
    _anonymous_ = ('bit',)
    _fields_ = [
        ('bit',   ControlStatusRegisterBits),
        ('value', ctypes.c_uint32)
    ]


class SelectRegisterBits(ctypes.LittleEndianStructure):
    """This class holds the different bit masks for the AP Select Register.

    Attributes:
      CTRLSEL: SW-DP debug port address bank select.
      RESERVED_A: reserved.
      APBANKSEL: selects the active four-word register window on the current
          access port.
      RESERVED_B: reserved.
      APSEL: selects the current access port.
    """
    _fields_ = [
        ('CTRLSEL',    ctypes.c_uint32, 1),
        ('RESERVED_A', ctypes.c_uint32, 3),
        ('APBANKSEL',  ctypes.c_uint32, 4),
        ('RESERVED_B', ctypes.c_uint32, 16),
        ('APSEL',      ctypes.c_uint32, 8)
    ]


class SelectRegisterFlags(ctypes.Union):
    """Mask for the select register bits.

    Attributes:
      value: the value stored in the mask.
    """
    _anonymous_ = ('bit',)
    _fields_ = [
        ('bit',   SelectRegisterBits),
        ('value', ctypes.c_uint32)
    ]


class MDMAPControlRegisterBits(ctypes.LittleEndianStructure):
    """This class holds the different bit masks for the MDM-AP Control
    Register.

    Attributes:
      flash_mass_erase: set to cause a mass erase, this is cleared
          automatically when a mass erase finishes.
      debug_disable: set to disable debug, clear to allow debug.
      debug_request: set to force the core to halt.
      sys_reset_request: set to force a system reset.
      core_hold_reset: set to suspend the core in reset at the end of reset
          sequencing.
      VLLDBGREQ: set to hold the system in reset after the next recovery from
          VLLSx (Very Low Leakage Stop).
      VLLDBGACK: set to release a system held in reset following a VLLSx
          (Very Low Leakage Stop) recovery.
      VLLSTATACK: set to acknowledge that the DAP LLS (Low Leakage Stop) and
          VLLS (Very Low Leakage Stop) status bits have read.
    """
    _fields_ = [
        ('flash_mass_erase',  ctypes.c_uint8, 1),
        ('debug_disable',     ctypes.c_uint8, 1),
        ('debug_request',     ctypes.c_uint8, 1),
        ('sys_reset_request', ctypes.c_uint8, 1),
        ('core_hold_reset',   ctypes.c_uint8, 1),
        ('VLLDBGREQ',         ctypes.c_uint8, 1),
        ('VLLDBGACK',         ctypes.c_uint8, 1),
        ('VLLSTATACK',        ctypes.c_uint8, 1)
    ]


class MDMAPControlRegisterFlags(ctypes.Union):
    """Mask for the MDM-AP control register bits.

    Attributes:
      value: the value stored in the mask.
    """
    _anonymous_ = ('bit',)
    _fields_ = [
        ('bit',   MDMAPControlRegisterBits),
        ('value', ctypes.c_uint8)
    ]


class MDMAPStatusRegisterBits(ctypes.LittleEndianStructure):
    """Holds the bit masks for the MDM-AP Status Register.

    Attributes:
      flash_mass_erase_ack: cleared after a system reset, indicates that a
          flash mass erase was acknowledged.
      flash_ready: indicates that flash has been initialized and can be
          configured.
      system_security: if set, system is secure and debugger cannot access the
          memory or system bus.
      system_reset: ``1`` if system is in reset, otherwise ``0``.
      mass_erase_enabled: ``1`` if MCU can be mass erased, otherwise ``0``.
      low_power_enabled: ``1`` if low power stop mode is enabled, otherwise ``0``.
      very_low_power_mode: ``1`` if device is in very low power mode.
      LLSMODEEXIT: indicates an exit from LLS mode has occurred.
      VLLSxMODEEXIT: indicates an exit from VLLSx mode has occured.
      core_halted; indicates core has entered debug halt mode.
      core_deep_sleep: indicates core has entered a low power mode.
      core_sleeping: indicates the core has entered a low power mode.

    Note:
      if ``core_sleeping & !core_deep_sleep``, then the core is in VLPW (very
      low power wait) mode, otherwise if ``core_sleeping & core_deep_sleep``,
      then it is in VLPS (very low power stop) mode.
    """
    _fields_ = [
        ('flash_mass_erase_ack',    ctypes.c_uint32, 1),
        ('flash_ready',             ctypes.c_uint32, 1),
        ('system_security',         ctypes.c_uint32, 1),
        ('system_reset',            ctypes.c_uint32, 1),
        ('RESERVED_A',              ctypes.c_uint32, 1),
        ('mass_erase_enabled',      ctypes.c_uint32, 1),
        ('backdoor_access_enabled', ctypes.c_uint32, 1),
        ('low_power_enabled',       ctypes.c_uint32, 1),
        ('very_low_power_mode',     ctypes.c_uint32, 1),
        ('LLSMODEEXIT',             ctypes.c_uint32, 1),
        ('VLLSxMODEEXIT',           ctypes.c_uint32, 1),
        ('RESERVED_B',              ctypes.c_uint32, 5),
        ('core_halted',             ctypes.c_uint32, 1),
        ('core_deep_sleep',         ctypes.c_uint32, 1),
        ('core_sleeping',           ctypes.c_uint32, 1),
        ('RESERVED_C',              ctypes.c_uint32, 13)
    ]


class MDMAPStatusRegisterFlags(ctypes.Union):
    """Mask for the MDM-AP status register bits.

    Attributes:
      value: the value stored in the mask.
    """
    _anonymous_ = ('bit',)
    _fields_ = [
        ('bit',   MDMAPStatusRegisterBits),
        ('value', ctypes.c_uint32)
    ]



# Copyright 2017 Square, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pylink.decorators as decorators

import mock

import threading
import unittest


class TestDecorators(unittest.TestCase):
    """Unit test for the `decorators` submodule."""

    def setUp(self):
        """Called before each test.

        Performs setup.

        Args:
          self (TestDecorators): the `TestDecorators` instance

        Returns:
          `None`
        """
        self.callback = mock.Mock()

    def tearDown(self):
        """Called after each test.

        Performs teardown.

        Args:
          self (TestDecorators): the `TestDecorators` instance

        Returns:
          `None`
        """
        pass

    def test_async_decorator_invalid(self):
        """Tests that the decorator raises an exception on invalid args.

        Args:
          self (TestDecorators): the `TestDecorators` instance

        Returns:
          `None`
        """
        @decorators.async_decorator
        def foo():
            return 4

        with self.assertRaises(TypeError):
            foo(callback='callback')

    def test_async_decorator_sync_call(self):
        """Tests that we can call the decorated method synchronously.

        Args:
          self (TestDecorators): the `TestDecorators` instance

        Returns:
          `None`
        """
        @decorators.async_decorator
        def foo():
            return 4

        self.assertEqual(4, foo())

    def test_async_decorator_join(self):
        """Tests that the returned object is a thread that can be joined and
        waited for termination.

        Args:
          self (TestDecorators): the `TestDecorators` instance

        Returns:
          `None`
        """
        @decorators.async_decorator
        def foo():
            return 4

        def callback(exception, result):
            return result

        thread = foo(callback=self.callback)
        self.assertTrue(isinstance(thread, threading.Thread))

        result = thread.join()
        self.assertTrue(isinstance(result, mock.Mock))
        self.callback.assert_called_with(None, 4)

        thread = foo(callback=callback)
        self.assertTrue(isinstance(thread, threading.Thread))

        result = thread.join()
        self.assertEqual(4, result)

    def test_async_decorator_exception(self):
        """Tests that exceptions raised in the async call are passed to the
        callback.

        Args:
          self (TestDecorators): the `TestDecorators` instance

        Returns:
          `None`
        """
        @decorators.async_decorator
        def failure():
            raise Exception('I HAVE FAILED!')

        thread = failure(callback=self.callback)
        thread.join()

        self.assertEqual(1, len(self.callback.call_args_list))
        self.assertTrue(self.callback.call_args is not None)

        exception = self.callback.call_args[0][0]
        self.assertTrue(isinstance(exception, BaseException))
        self.assertEqual('I HAVE FAILED!', str(exception))

        res = self.callback.call_args[0][1]
        self.assertEqual(None, res)


if __name__ == '__main__':
    unittest.main()



# Copyright 2017 Square, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pylink.protocols.swd as swd
import pylink.util

import mock

import unittest


class TestSerialWireDebug(unittest.TestCase):
    """Tests the `protocols.swd` submodule."""

    def setUp(self):
        """Called before each test.

        Performs setup.

        Args:
          self (TestSerialWireDebug): the `TestSerialWireDebug` instance

        Returns:
          `None`
        """
        pass

    def tearDown(self):
        """Called after each test.

        Performs teardown.

        Args:
          self (TestSerialWireDebug): the `TestSerialWireDebug` instance

        Returns:
          `None`
        """
        pass

    def test_swd_response_ack(self):
        """Tests creating an ACK response.

        Args:
          self (TestSerialWireDebug): the `TestSerialWireDebug` instance

        Returns:
          `None`
        """
        response = swd.Response(swd.Response.STATUS_ACK)
        self.assertTrue(response.ack())
        self.assertFalse(response.wait())
        self.assertFalse(response.fault())
        self.assertFalse(response.invalid())

    def test_swd_response_wait(self):
        """Tests creating a WAIT response.

        Args:
          self (TestSerialWireDebug): the `TestSerialWireDebug` instance

        Returns:
          `None`
        """
        response = swd.Response(swd.Response.STATUS_WAIT)
        self.assertTrue(response.wait())
        self.assertFalse(response.ack())
        self.assertFalse(response.fault())
        self.assertFalse(response.invalid())

    def test_swd_response_fault(self):
        """Tests creating a FAULT response.

        Args:
          self (TestSerialWireDebug): the `TestSerialWireDebug` instance

        Returns:
          `None`
        """
        response = swd.Response(swd.Response.STATUS_FAULT)
        self.assertTrue(response.fault())
        self.assertFalse(response.wait())
        self.assertFalse(response.ack())
        self.assertFalse(response.invalid())

    def test_swd_response_invalid(self):
        """Tests creating an invalid response.

        Args:
          self (TestSerialWireDebug): the `TestSerialWireDebug` instance

        Returns:
          `None`
        """
        response = swd.Response(swd.Response.STATUS_INVALID)
        self.assertTrue(response.invalid())
        self.assertFalse(response.fault())
        self.assertFalse(response.wait())
        self.assertFalse(response.ack())

    def test_swd_read_request_initialize(self):
        """Tests creating a SWD Read Request.

        When a SWD Read Request is created, there is a structure specifying
        what the underlying bits should look like.  This test verifies a
        number of different valid bitfields.

        Args:
          self (TestSerialWireDebug): the `TestSerialWireDebug` instance

        Returns:
          `None`
        """
        values = [165, 141, 149, 189, 165]
        for (index, value) in enumerate(values):
            request = swd.ReadRequest(index, ap=False)
            self.assertEqual(value, request.value)

        values = [135, 175, 183, 159, 135]
        for (index, value) in enumerate(values):
            request = swd.ReadRequest(index, ap=True)
            self.assertEqual(value, request.value)

    def test_swd_read_request_send_nack(self):
        """Tests sending a SWD Read Request that is NACK'd.

        Args:
          self (TestSerialWireDebug): the `TestSerialWireDebug` instance

        Returns:
          `None`
        """
        request = swd.ReadRequest(0, True)

        ack = 1
        status = swd.Response.STATUS_WAIT
        data = 2

        mock_jlink = mock.Mock()
        mock_jlink.swd_write.return_value = ack
        mock_jlink.swd_read8.return_value = status
        mock_jlink.swd_read32.return_value = data

        response = request.send(mock_jlink)

        self.assertFalse(response.ack())
        self.assertTrue(response.wait())

        self.assertEqual(2, mock_jlink.swd_write8.call_count)
        mock_jlink.swd_write8.assert_any_call(0xFF, request.value)  # data command
        mock_jlink.swd_write8.assert_any_call(0xFC, 0x0)  # status command

        self.assertEqual(1, mock_jlink.swd_write32.call_count)
        mock_jlink.swd_write32.assert_any_call(0x0, 0x0)

        self.assertEqual(1, mock_jlink.swd_write.call_count)
        mock_jlink.swd_write.assert_any_call(0x0, 0x0, 3)  # ack

        self.assertEqual(1, mock_jlink.swd_read8.call_count)
        mock_jlink.swd_read8.assert_any_call(ack)  # status read

        self.assertEqual(1, mock_jlink.swd_read32.call_count)
        mock_jlink.swd_read32.assert_any_call(ack + 3)  # data read

    def test_swd_read_request_send_ack(self):
        """Tests sending a SWD Read Request that is ACK'd.

        Args:
          self (TestSerialWireDebug): the `TestSerialWireDebug` instance

        Returns:
          `None`
        """
        request = swd.ReadRequest(0, True)

        ack = 1
        status = swd.Response.STATUS_ACK
        data = 1

        mock_jlink = mock.Mock()
        mock_jlink.swd_write.return_value = ack
        mock_jlink.swd_read8.return_value = status
        mock_jlink.swd_read32.return_value = data

        response = request.send(mock_jlink)

        self.assertTrue(response.ack())

        self.assertEqual(2, mock_jlink.swd_write8.call_count)
        mock_jlink.swd_write8.assert_any_call(0xFF, request.value)  # data command
        mock_jlink.swd_write8.assert_any_call(0xFC, 0x0)  # status command

        self.assertEqual(1, mock_jlink.swd_write32.call_count)
        mock_jlink.swd_write32.assert_any_call(0x0, 0x0)

        self.assertEqual(1, mock_jlink.swd_write.call_count)
        mock_jlink.swd_write.assert_any_call(0x0, 0x0, 3)  # ack

        self.assertEqual(2, mock_jlink.swd_read8.call_count)
        mock_jlink.swd_read8.assert_any_call(ack)  # status read
        mock_jlink.swd_read8.assert_any_call(ack + 35)  # parity check

        self.assertEqual(1, mock_jlink.swd_read32.call_count)
        mock_jlink.swd_read32.assert_any_call(ack + 3)  # data read

    def test_swd_read_request_send_ack_parity_mismatch(self):
        """Tests sending a SWD Request that is ACK'd, but the parity is wrong.

        When a SWD Read Request reads data from the target, their is a parity
        field that is set, and can be is to verify that the data is valid.  In
        this test, the parity check fails.

        Args:
          self (TestSerialWireDebug): the `TestSerialWireDebug` instance

        Returns:
          `None`
        """
        request = swd.ReadRequest(0, True)

        ack = 1
        status = swd.Response.STATUS_ACK
        data = 3

        mock_jlink = mock.Mock()
        mock_jlink.swd_write.return_value = ack
        mock_jlink.swd_read8.return_value = status
        mock_jlink.swd_read32.return_value = data

        response = request.send(mock_jlink)

        self.assertFalse(response.ack())
        self.assertTrue(response.invalid())

        self.assertEqual(2, mock_jlink.swd_write8.call_count)
        mock_jlink.swd_write8.assert_any_call(0xFF, request.value)  # data command
        mock_jlink.swd_write8.assert_any_call(0xFC, 0x0)  # status command

        self.assertEqual(1, mock_jlink.swd_write32.call_count)
        mock_jlink.swd_write32.assert_any_call(0x0, 0x0)

        self.assertEqual(1, mock_jlink.swd_write.call_count)
        mock_jlink.swd_write.assert_any_call(0x0, 0x0, 3)  # ack

        self.assertEqual(2, mock_jlink.swd_read8.call_count)
        mock_jlink.swd_read8.assert_any_call(ack)  # status read
        mock_jlink.swd_read8.assert_any_call(ack + 35)  # parity check

        self.assertEqual(1, mock_jlink.swd_read32.call_count)
        mock_jlink.swd_read32.assert_any_call(ack + 3)  # data read

    def test_swd_write_request_initialize(self):
        """Tests creating a SWD Write Request.

        When a SWD Write Request is created, there is a structure specifying
        what the underlying bits should look like.  This test verifies a number
        of different valid bitfields.

        Args:
          self (TestSerialWireDebug): the `TestSerialWireDebug` instance

        Returns:
          `None`
        """
        data = 4
        values = [129, 169, 177, 153]
        for (index, value) in enumerate(values):
            request = swd.WriteRequest(index, data=data, ap=False)
            self.assertEqual(value, request.value)

        values = [163, 139, 147, 187]
        for (index, value) in enumerate(values):
            request = swd.WriteRequest(index, data=data, ap=True)
            self.assertEqual(value, request.value)

    def test_swd_write_request_send(self):
        """Tests sending a SWD Read Request.

        Args:
          self (TestSerialWireDebug): the `TestSerialWireDebug` instance

        Returns:
          `None`
        """
        data = 2
        parity = pylink.util.calculate_parity(data)
        request = swd.WriteRequest(0, True, data)

        ack = 2
        mock_jlink = mock.Mock()
        mock_jlink.swd_write.return_value = ack
        mock_jlink.swd_read8.return_value = 1

        response = request.send(mock_jlink)
        self.assertTrue(response.ack())

        self.assertEqual(2, mock_jlink.swd_write.call_count)
        mock_jlink.swd_write.assert_any_call(0x0, 0x0, 3)  # Ack
        mock_jlink.swd_write.assert_any_call(0x0, 0x0, 2)  # Turnaround

        self.assertEqual(2, mock_jlink.swd_write8.call_count)
        mock_jlink.swd_write8.assert_any_call(0xFF, request.value)
        mock_jlink.swd_write8.assert_any_call(0xFF, parity)

        self.assertEqual(1, mock_jlink.swd_write32.call_count)
        mock_jlink.swd_write32.assert_called_once_with(0xFFFFFFFF, data)

        self.assertEqual(1, mock_jlink.swd_read8.call_count)
        mock_jlink.swd_read8.assert_called_once_with(ack)


if __name__ == '__main__':
    unittest.main()



# Copyright 2017 Square, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pylink

import argparse
import logging
import os
import sys
try:
    from six.moves import with_metaclass
except (AttributeError, ImportError):
    from six import with_metaclass


class CommandMeta(type):
    """Meta-class for a command."""

    registry = {}

    def __new__(cls, name, parents, dct):
        """Creates a new Command class and validates it.

        Args:
          cls (Class): the class object being created
          name (name): the name of the class being created
          parents (list): list of parent classes
          dct (dictionary): class attributes

        Returns:
          ``Class``
        """
        newClass = super(CommandMeta, cls).__new__(cls, name, parents, dct)

        if name != 'Command':
            for attribute in ['name', 'description', 'help']:
                if attribute not in dct or dct[attribute] is None:
                    raise ValueError('%s cannot be None.' % attribute)
            CommandMeta.registry[name] = newClass

        return newClass


class Command(with_metaclass(CommandMeta)):
    """Base command-class.

    All commands should inherit from this class.

    Attributes:
      name: name of the command, should be unique.
      description: command description string.
      help: command help string.
    """
    name = None
    description = None
    help = None

    def create_jlink(self, args):
        """Creates an instance of a J-Link from the given arguments.

        Args:
          self (Command): the ``Command`` instance
          args (Namespace): arguments to construct the ``JLink`` instance from

        Returns:
          An instance of a ``JLink``.
        """
        jlink = pylink.JLink()
        jlink.open(args.serial_no, args.ip_addr)

        if hasattr(args, 'tif') and args.tif is not None:
            if args.tif.lower() == 'swd':
                jlink.set_tif(pylink.JLinkInterfaces.SWD)
            else:
                jlink.set_tif(pylink.JLinkInterfaces.JTAG)

        if hasattr(args, 'device') and args.device is not None:
            jlink.connect(args.device)

        return jlink

    def add_common_arguments(self, parser, has_device=False):
        """Adds common arguments to the given parser.

        Common arguments for a J-Link command are the target interface, and
        J-Link serial number or IP address.

        Args:
          self (Command): the ``Command`` instance
          parser (argparse.ArgumentParser): the parser to add the arguments to
          has_device (bool): boolean indicating if it has the device argument

        Returns:
          ``None``
        """
        if has_device:
            parser.add_argument('-t', '--tif', required=True,
                                type=str.lower, choices=['jtag', 'swd'],
                                help='target interface (JTAG | SWD)')
            parser.add_argument('-d', '--device', required=True,
                                help='specify the target device name')

        group = parser.add_mutually_exclusive_group(required=False)
        group.add_argument('-s', '--serial', dest='serial_no',
                           help='specify the J-Link serial number')
        group.add_argument('-i', '--ip_addr', dest='ip_addr',
                           help='J-Link IP address')

        return None

    def add_arguments(self, parser):
        """Adds arguments to the given parser.

        Not implemented.  The derived class must implement this.

        Args:
          self (Command): the ``Command`` instance
          parser (argparse.ArgumentParser): the parser to add the arguments to

        Returns:
          ``None``

        Raises:
          NotImplementedError: always.
        """
        raise NotImplementedError('%s not implemented.' % self.__class__.__name__)

    def run(self, args):
        """Runs the command.

        Not implemented.  The derived class must implement this.

        Args:
          self (Command): the ``Command`` instance
          args (Namespace): the arguments passed on the command-line

        Returns:
          ``None``

        Raises:
          NotImplementedError: always.
        """
        raise NotImplementedError('%s not implemented.' % self.__class__.__name__)


class EraseCommand(Command):
    """Defines the erase command for erasing a device."""
    name = 'erase'
    description = 'Erases the target device.'
    help = 'erases the device connected to the J-Link'

    def add_arguments(self, parser):
        """Adds the erase command arguments to the parser.

        Args:
          self (EraseCommand): the ``EraseCommand`` instance
          parser (argparse.ArgumentParser): the parser to add the arguments to

        Returns:
          ``None``
        """
        return self.add_common_arguments(parser, True)

    def run(self, args):
        """Erases the device connected to the J-Link.

        Args:
          self (EraseCommand): the ``EraseCommand`` instance
          args (Namespace): the arguments passed on the command-line

        Returns:
          ``None``
        """
        jlink = self.create_jlink(args)
        erased = jlink.erase()
        print('Bytes Erased: %d' % erased)


class FlashCommand(Command):
    """Defines the flash command for flashing a device."""
    name = 'flash'
    description = 'Flashes firmware from a file to a device connected to a J-Link.'
    help = 'flash a device connected to the J-Link'

    def add_arguments(self, parser):
        """Adds the flash command arguments to the parser.

        Args:
          self (FlashCommand): the ``FlashCommand`` instance
          parser (argparse.ArgumentParser): the parser to add the arguments to

        Returns:
          ``None``
        """
        parser.add_argument('-a', '--addr', type=int, default=0,
                            help='start address to flash from')
        parser.add_argument('file', nargs=1, help='file to flash onto device')
        return self.add_common_arguments(parser, True)

    def run(self, args):
        """Flashes the device connected to the J-Link.

        Args:
          self (FlashCommand): the ``FlashCommand`` instance
          args (Namespace): the arguments passed on the command-line

        Returns:
          ``None``
        """
        kwargs = {}
        kwargs['path'] = args.file[0]
        kwargs['addr'] = args.addr
        kwargs['on_progress'] = pylink.util.flash_progress_callback

        jlink = self.create_jlink(args)
        _ = jlink.flash_file(**kwargs)
        print('Flashed device successfully.')


class UnlockCommand(Command):
    """Command for unlocking a device."""
    name = 'unlock'
    description = (
        'Unlocks a device connected to a J-Link.  '
        'Note that this will erase the device.'
    )
    help = 'unlock a connected device'

    def add_arguments(self, parser):
        """Adds the unlock command arguments to the parser.

        Args:
          self (UnlockCommand): the ``UnlockCommand`` instance
          parser (argparse.ArgumentParser): the parser to add the arguments to

        Returns:
          ``None``
        """
        parser.add_argument('name', nargs=1, choices=['kinetis'],
                            help='name of MCU to unlock')
        return self.add_common_arguments(parser, True)

    def run(self, args):
        """Unlocks the target device.

        Args:
          self (UnlockCommand): the ``UnlockCommand`` instance
          args (Namespace): the arguments passed on the command-line

        Returns:
          ``None``
        """
        jlink = self.create_jlink(args)
        mcu = args.name[0].lower()
        if pylink.unlock(jlink, mcu):
            print('Successfully unlocked device!')
        else:
            print('Failed to unlock device!')


class LicenseCommand(Command):
    """Command for managing the J-Link's licenses."""
    name = 'license'
    description = 'Manage the licenses of the J-Link.'
    help = 'manage the licenses of your J-Link'

    def add_arguments(self, parser):
        """Adds the license command arguments to the parser.

        Args:
          self (LicenseCommand): the ``LicenseCommand`` instance
          parser (argparse.ArgumentParser): the parser to add the arguments to

        Returns:
          ``None``
        """
        group = parser.add_mutually_exclusive_group(required=True)
        group.add_argument('-l', '--list', action='store_true',
                           help='list the licenses of the J-Link')
        group.add_argument('-a', '--add', dest='add',
                           help='add a custom license to the J-Link')
        group.add_argument('-e', '--erase', action='store_true',
                           help='erase the custom licenses on the J-Link')
        return self.add_common_arguments(parser, False)

    def run(self, args):
        """Runs the license command.

        Args:
          self (LicenseCommand): the ``LicenseCommand`` instance
          args (Namespace): the arguments passed on the command-line

        Returns:
          ``None``
        """
        jlink = self.create_jlink(args)
        if args.list:
            print('Built-in Licenses: %s' % ', '.join(jlink.licenses.split(',')))
            print('Custom Licenses: %s' % ', '.join(jlink.custom_licenses.split(',')))
        elif args.add is not None:
            if jlink.add_license(args.add):
                print('Successfully added license.')
            else:
                print('License already exists.')
        elif args.erase:
            if jlink.erase_licenses():
                print('Successfully erased all custom licenses.')
            else:
                print('Failed to erase custom licenses.')


class InfoCommand(Command):
    """Command for getting information about the hardware / DLL."""
    name = 'info'
    description = 'Get information about the J-Link.'
    help = 'get information about the J-Link'

    def add_arguments(self, parser):
        """Adds the information commands to the parser.

        Args:
          self (InfoCommand): the ``InfoCommand`` instance
          parser (argparse.ArgumentParser): the parser to add the arguments to

        Returns:
          ``None``
        """
        parser.add_argument('-p', '--product', action='store_true',
                            help='print the production information')
        parser.add_argument('-j', '--jtag', action='store_true',
                            help='print the JTAG pin status')
        return self.add_common_arguments(parser, False)

    def run(self, args):
        """Runs the information command.

        Args:
          self (InfoCommand): the ``InfoCommand`` instance
          args (Namespace): the arguments passed on the command-line

        Returns:
          ``None``
        """
        jlink = self.create_jlink(args)
        if args.product:
            print('Product: %s' % jlink.product_name)

            manufacturer = 'SEGGER' if jlink.oem is None else jlink.oem
            print('Manufacturer: %s' % manufacturer)

            print('Hardware Version: %s' % jlink.hardware_version)
            print('Firmware: %s' % jlink.firmware_version)
            print('DLL Version: %s' % jlink.version)
            print('Features: %s' % ', '.join(jlink.features))
        elif args.jtag:
            status = jlink.hardware_status
            print('TCK Pin Status: %d' % status.tck)
            print('TDI Pin Status: %d' % status.tdi)
            print('TDO Pin Status: %d' % status.tdo)
            print('TMS Pin Status: %d' % status.tms)
            print('TRES Pin Status: %d' % status.tres)
            print('TRST Pin Status: %d' % status.trst)


class EmulatorCommand(Command):
    """Command for querying about emulator support, or connected emulators."""
    name = 'emulator'
    description = 'Query for information about emulators or support.'
    help = 'query for information about emulators or support'

    def add_arguments(self, parser):
        """Adds the arguments for the emulator command.

        Args:
          self (EmulatorCommand): the ``EmulatorCommand`` instance
          parser (argparse.ArgumentParser): parser to add the commands to

        Returns:
          ``None``
        """
        group = parser.add_mutually_exclusive_group(required=True)
        group.add_argument('-l', '--list', nargs='?',
                           type=str.lower, default='_',
                           choices=['usb', 'ip'],
                           help='list all the connected emulators')
        group.add_argument('-s', '--supported', nargs=1,
                           help='query whether a device is supported')
        group.add_argument('-t', '--test', action='store_true',
                           help='perform a self-test')
        return None

    def run(self, args):
        """Runs the emulator command.

        Args:
          self (EmulatorCommand): the ``EmulatorCommand`` instance
          args (Namespace): arguments to parse

        Returns:
          ``None``
        """
        jlink = pylink.JLink()

        if args.test:
            if jlink.test():
                print('Self-test succeeded.')
            else:
                print('Self-test failed.')
        elif args.list is None or args.list in ['usb', 'ip']:
            host = pylink.JLinkHost.USB_OR_IP
            if args.list == 'usb':
                host = pylink.JLinkHost.USB
            elif args.list == 'ip':
                host = pylink.JLinkHost.IP

            emulators = jlink.connected_emulators(host)
            for (index, emulator) in enumerate(emulators):
                if index > 0:
                    print('')

                print('Product Name: %s' % emulator.acProduct.decode())
                print('Serial Number: %s' % emulator.SerialNumber)

                usb = bool(emulator.Connection)
                if not usb:
                    print('Nickname: %s' % emulator.acNickname.decode())
                    print('Firmware: %s' % emulator.acFWString.decode())

                print('Connection: %s' % ('USB' if usb else 'IP'))

                if not usb:
                    print('IP Address: %s' % emulator.aIPAddr)
        elif args.supported is not None:
            device = args.supported[0]
            try:
                index = jlink.get_device_index(device)
            except pylink.errors.JLinkException:
                print('%s is not supported :(' % device)
                return None

            found_device = jlink.supported_device(index)

            print('Device Name: %s' % device)
            print('Core ID: %s' % found_device.CoreId)
            print('Flash Address: %s' % found_device.FlashAddr)
            print('Flash Size: %s bytes' % found_device.FlashSize)
            print('RAM Address: %s' % found_device.RAMAddr)
            print('RAM Size: %s bytes' % found_device.RAMSize)
            print('Manufacturer: %s' % found_device.manufacturer)

        return None


class FirmwareCommand(Command):
    """Command for upgrading and downgrading J-Link firmware."""
    name = 'firmware'
    description = 'Modify the J-Link firmware.'
    help = 'modify the J-Link firmware'

    def add_arguments(self, parser):
        """Adds the arguments for the firmware command.

        Args:
          self (FirmwareCommand): the ``FirmwareCommand`` instance
          parser (argparse.ArgumentParser): parser to add the commands to

        Returns:
          ``None``
        """
        group = parser.add_mutually_exclusive_group(required=True)
        group.add_argument('-d', '--downgrade', action='store_true',
                           help='downgrade the J-Link firmware')
        group.add_argument('-u', '--upgrade', action='store_true',
                           help='upgrade the J-Link firmware')
        return self.add_common_arguments(parser, False)

    def run(self, args):
        """Runs the firmware command.

        Args:
          self (FirmwareCommand): the ``FirmwareCommand`` instance
          args (Namespace): arguments to parse

        Returns:
          ``None``
        """
        jlink = self.create_jlink(args)
        if args.downgrade:
            if not jlink.firmware_newer():
                print('DLL firmware is not older than J-Link firmware.')
            else:
                jlink.invalidate_firmware()

                try:
                    # Change to the firmware of the connected DLL.
                    jlink.update_firmware()
                except pylink.JLinkException as e:
                    # On J-Link versions < 5.0.0, an exception will be thrown as
                    # the connection will be lost, so we have to re-establish.
                    jlink = self.create_jlink(args)

                print('Firmware Downgraded: %s' % jlink.firmware_version)
        elif args.upgrade:
            if not jlink.firmware_outdated():
                print('DLL firmware is not newer than J-Link firmware.')
            else:
                try:
                    # Upgrade the firmware.
                    jlink.update_firmware()
                except pylink.JLinkException as e:
                    # On J-Link versions < 5.0.0, an exception will be thrown as
                    # the connection will be lost, so we have to re-establish.
                    jlink = self.create_jlink(args)
                print('Firmware Updated: %s' % jlink.firmware_version)

        return None


def commands():
    """Returns the program commands.

    Returns:
      A list of commands.
    """
    return map(lambda c: c(), CommandMeta.registry.values())


def create_parser():
    """Builds the command parser.

    This needs to be exported in order for Sphinx to document it correctly.

    Returns:
      An instance of an ``argparse.ArgumentParser`` that parses all the
      commands supported by the PyLink CLI.
    """
    parser = argparse.ArgumentParser(prog=pylink.__title__,
                                     description=pylink.__description__,
                                     epilog=pylink.__copyright__)
    parser.add_argument('--version', action='version',
                        version='%(prog)s ' + pylink.__version__)
    parser.add_argument('-v', '--verbose', action='count', default=0,
                        help='increase output verbosity')

    kwargs = {}
    kwargs['title'] = 'command'
    kwargs['description'] = 'specify subcommand to run'
    kwargs['help'] = 'subcommands'
    subparsers = parser.add_subparsers(**kwargs)

    for command in commands():
        kwargs = {}
        kwargs['name'] = command.name
        kwargs['description'] = command.description
        kwargs['help'] = command.help
        subparser = subparsers.add_parser(**kwargs)
        subparser.set_defaults(command=command.run)
        command.add_arguments(subparser)

    return parser


def main(args=None):
    """Main command-line interface entrypoint.

    Runs the given subcommand or argument that were specified.  If not given a
    ``args`` parameter, assumes the arguments are passed on the command-line.

    Args:
      args (list): list of command-line arguments

    Returns:
      Zero on success, non-zero otherwise.
    """
    if args is None:
        args = sys.argv[1:]

    parser = create_parser()
    args = parser.parse_args(args)

    if args.verbose >= 2:
        level = logging.DEBUG
    elif args.verbose >= 1:
        level = logging.INFO
    else:
        level = logging.WARNING

    logging.basicConfig(level=level)

    try:
        if hasattr(args, 'command'):
            args.command(args)
        else:
            # Python 3 argparse won't create the command attribute and an
            # AttributeError will be raised if no commands are specified. Note:
            # Python 3.7 added support for subparsers being required.  Emulate
            # Python 2 error here for consistent behavior.
            parser.error('too few arguments')
    except pylink.JLinkException as e:
        sys.stderr.write('Error: %s%s' % (str(e), os.linesep))
        return 1

    return 0


if __name__ == '__main__':
    exit(main())



# Copyright 2017 Square, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pylink

import behave

try:
    import StringIO
except ImportError:
    import io as StringIO
import sys
import time


def create_jlink(context):
    """Creates an instance of a J-Link, attaching it to the context.

    Args:
      context (Context): the ``Context`` instance

    Returns:
      ``JLink``
    """
    context.log = StringIO.StringIO()
    context.jlink = pylink.JLink(log=context.log.write,
                                 detailed_log=context.log.write)

    emulators = context.jlink.connected_emulators()
    if len(emulators) > 0:
        serial_no = emulators[0].SerialNumber
        context.jlink.disable_dialog_boxes()
        context.jlink.open(serial_no=serial_no)

    return context.jlink


def reset_jlink(context):
    """Resets the context's J-Link instance by destroying it.

    Args:
      context (Context): the ``Context`` instance

    Returns:
      ``None``
    """
    context.jlink.close()
    return create_jlink(context)


def extract_tag(tags, prefix):
    """Extracts a tag and its value.

    Consumes a tag prefix, and extracts the value corresponding to it.

    Args:
      tags (list): the list of tags to check
      prefix (str): the tag prefix

    Returns:
      A tuple of tag and tag value.
    """
    for tag in tags:
        if tag.startswith(prefix):
            suffix = tag[len(prefix):]
            if len(suffix) <= 1 or not suffix[0] == '=':
                raise ValueError('Invalid tag: \'%s\'' % tag)
            return prefix, suffix[1:]
    return None, None


def should_exclude(context, section):
    """Returns whether the given section should be skipped.

    Args:
      context (Context): the ``Context`` instance
      section: the section to skip

    Returns:
      ``True`` if section should be skipped, otherwise ``False``.
    """
    if len(section.tags) == 0:
        return False

    if 'connection' in section.tags:
        if not context.jlink.connected():
            return True
    elif 'nix' in section.tags:
        return sys.platform.startswith('win')

    return False


def before_all(context):
    """Runs before all tests.

    Syncs the J-Link firmware before running tests.

    Args:
      context (Context): the ``Context`` instance

    Returns:
      ``None``
    """
    jlink = create_jlink(context)
    jlink.sync_firmware()

    print('using J-Link Firmware: %s' % jlink.firmware_version)
    print('using J-Link DLL: %s' % jlink.version)
    print('')


def after_all(context):
    """Runs after all tests.

    Args:
      context (Context): the ``Context`` instance

    Returns:
      ``None``
    """
    pass


def before_feature(context, feature):
    """Runs before each feature.

    Args:
      context (Context): the ``Context`` instance
      feature (Feature): the feature that is next to run

    Returns:
      ``None``
    """
    reset_jlink(context)

    if should_exclude(context, feature):
        feature.skip(reason='Windows platform or no J-Link connection in Feature')


def after_feature(context, feature):
    """Runs after each feature.

    Args:
      context (Context): the ``Context`` instance
      feature (Feature): the feature that has just ran

    Returns:
      ``None``
    """
    pass


def before_scenario(context, scenario):
    """Runs before each scenario.

    Args:
      context (Context): the ``Context`` instance
      scenario (Scenario): the scenario that is next to run

    Returns:
      ``None``
    """
    reset_jlink(context)

    if should_exclude(context, scenario):
        scenario.skip(reason='Windows platform or no J-Link connection in Scenario')

    context.scenario = scenario


def after_scenario(context, scenario):
    """Runs after each scenario.

    Args:
      context (Context): the ``Context`` instance
      scenario (Scenario): the scenario that has just ran

    Returns:
      ``None``
    """
    if getattr(scenario, 'skip_reason', None) is not None:
        sys.stdout.write('    Skipped Scenario.  Reason: %s\n' % scenario.skip_reason)

    jlink = context.jlink
    if jlink.connected():
        if jlink.swo_enabled():
            jlink.swo_stop()

        if len(jlink.custom_licenses) > 0:
            jlink.erase_licenses()

    jlink.close()


def before_tag(context, tag):
    """Runs before each section with the given tag.

    Args:
      context (Context): the ``Context`` instance
      tag (Tag): the section's tag

    Returns:
      ``None``
    """
    pass


def after_tag(context, tag):
    """Runs after each section with the given tag.

    Args:
      context (Context): the ``Context`` instance
      tag (Tag): the section's tag

    Returns:
      ``None``
    """
    pass


def before_step(context, step):
    """Runs before each step.

    Args:
      context (Context): the ``Context`` instance
      step (Step): the step that is next to run

    Returns:
      ``None``
    """
    context.step = step


def after_step(context, step):
    """Runs after each step.

    Args:
      context (Context): the ``Context`` instance
      step (Step): the step that has just ran

    Returns:
      ``None``
    """
    pass



# Copyright 2017 Square, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import ctypes
import math


BITS_PER_BYTE = 8


def pack_size(value):
    """Returns the number of bytes required to represent a given value.

    Args:
      value (int): the natural number whose size to get

    Returns:
      The minimal number of bytes required to represent the given integer.

    Raises:
      ValueError: if ``value < 0``.
      TypeError: if ``value`` is not a number.
    """
    if value == 0:
        return 1
    elif value < 0:
        raise ValueError('Expected non-negative integer.')
    return int(math.log(value, 256)) + 1


def pack(value, nbits=None):
    """Packs a given value into an array of 8-bit unsigned integers.

    If ``nbits`` is not present, calculates the minimal number of bits required
    to represent the given ``value``.  The result is little endian.

    Args:
      value (int): the integer value to pack
      nbits (int): optional number of bits to use to represent the value

    Returns:
      An array of ``ctypes.c_uint8`` representing the packed ``value``.

    Raises:
      ValueError: if ``value < 0`` and ``nbits`` is ``None`` or ``nbits <= 0``.
      TypeError: if ``nbits`` or ``value`` are not numbers.
    """
    if nbits is None:
        nbits = pack_size(value) * BITS_PER_BYTE
    elif nbits <= 0:
        raise ValueError('Given number of bits must be greater than 0.')

    buf_size = int(math.ceil(nbits / float(BITS_PER_BYTE)))
    buf = (ctypes.c_uint8 * buf_size)()

    for (idx, _) in enumerate(buf):
        buf[idx] = (value >> (idx * BITS_PER_BYTE)) & 0xFF

    return buf



# Copyright 2017 Square, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pylink

import behave

import time


@behave.when('my {device} device is locked')
def step_device_locked(context, device):
    """Locks the specified device.

    Note:
      Currently only supports Kinetis.

    Args:
      context (Context): the ``Context`` instance
      device (str): the type of device to lock

    Returns:
      ``None``
    """
    assert device.lower() == 'kinetis'

    context.device = device

    # Security bytes.  These bytes control the device security.  We want to
    # secure the device, but still allow it to be mass-erased (otherwise the
    # device will be permanently locked.
    fcf_offset = 0x400
    fcf_bytes = [
        0xFF,  # Backdoor comparison key (unused, 8 bytes long)
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,
        0xFF,  # Flash protection settings (4 bytes long)
        0xFF,
        0xFF,
        0xFF,
        0x03,  # FSEC register - mass erase enabled, flash security enabled
        0x07,  # FOPT register - NMI enabled, EzPort enabled, normal boot
        0xFF,  # Data flash protection settings (unused)
        0xFF,  # EEPROM settings (unused)
    ]
    assert context.jlink.flash_write(fcf_offset, fcf_bytes) >= 0

    # Reset the device for the changes to take.
    context.jlink.reset()
    context.jlink.halt()

    try:
        # This write should fail as the device is flash protected now.
        context.jlink.flash_write(fcf_offset, fcf_bytes)
    except Exception as e:
        pass
    else:
        assert False

    time.sleep(1)


@behave.when('I unlock it')
def step_unlock_device(context):
    """Unlocks the connected device.

    Args:
      context (Context): the ``Context`` instance

    Returns:
      ``None``
    """
    jlink = context.jlink
    device = context.device

    assert jlink.halt()

    jlink.set_reset_strategy(pylink.JLinkResetStrategyCortexM3.RESETPIN)
    jlink.reset()
    jlink.set_reset_strategy(pylink.JLinkResetStrategyCortexM3.NORMAL)

    assert jlink.target_connected()
    assert jlink.unlock()


@behave.then('I should be able to write to flash')
def step_check_write_to_flash(context):
    """Checks that the target can be written to over flash.

    Args:
      context (Context): the ``Context`` instance

    Returns:
      ``None``
    """
    jlink = context.jlink
    assert jlink.target_connected()
    assert context.jlink.flash_write(0x40C, [0x2]) >= 0



# -*- coding: utf-8 -*-
# Copyright 2017 Square, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from . import enums

import platform
import sys


def is_integer(val):
    """Returns whether the given value is an integer.

    Args:
      val (object): value to check

    Returns:
      ``True`` if the given value is an integer, otherwise ``False``.
    """
    try:
        val += 1
    except TypeError:
        return False
    return True


def is_natural(val):
    """Returns whether the given value is a natrual number.

    Args:
      val (object): value to check

    Returns:
      ``True`` if the given value is a natural number, otherwise ``False``.
    """
    return is_integer(val) and (val >= 0)


def is_os_64bit():
    """Returns whether the current running platform is 64bit.

    Returns:
      ``True`` if the platform is 64bit, otherwise ``False``.
    """
    return platform.machine().endswith('64')


def noop(*args, **kwargs):
    """No-op.  Does nothing.

    Args:
      args: list of arguments
      kwargs: keyword arguments dictionary

    Returns:
      ``None``
    """
    pass


def unsecure_hook_dialog(title, msg, flags):
    """No-op that ignores the dialog.

    Args:
      title (str): title of the unsecure dialog
      msg (str): text of the unsecure dialog
      flags (int): flags specifying which values can be returned

    Returns:
      ``enums.JLinkFlags.DLG_BUTTON_NO``
    """
    return enums.JLinkFlags.DLG_BUTTON_NO


def progress_bar(iteration,
                 total,
                 prefix=None,
                 suffix=None,
                 decs=1,
                 length=100):
    """Creates a console progress bar.

    This should be called in a loop to create a progress bar.

    See `StackOverflow <http://stackoverflow.com/questions/3173320/>`__.

    Args:
      iteration (int): current iteration
      total (int): total iterations
      prefix (str): prefix string
      suffix (str): suffix string
      decs (int): positive number of decimals in percent complete
      length (int): character length of the bar

    Returns:
      ``None``

    Note:
      This function assumes that nothing else is printed to the console in the
      interim.
    """
    if prefix is None:
        prefix = ''

    if suffix is None:
        suffix = ''

    format_str = '{0:.' + str(decs) + 'f}'
    percents = format_str.format(100 * (iteration / float(total)))
    filled_length = int(round(length * iteration / float(total)))
    bar = '█' * filled_length + '-' * (length - filled_length)

    prefix, suffix = prefix.strip(), suffix.strip()

    sys.stdout.write('\r%s |%s| %s%s %s' % (prefix, bar, percents, '%', suffix))
    sys.stdout.flush()

    if iteration == total:
        sys.stdout.write('\n')
        sys.stdout.flush()

    return None


def flash_progress_callback(action, progress_string, percentage):
    """Callback that can be used with ``JLink.flash()``.

    This callback generates a progress bar in the console to show the progress
    of each of the steps of the flash.

    Args:
      action (str): the current action being invoked
      progress_string (str): the current step in the progress
      percentage (int): the percent to which the current step has been done

    Returns:
      ``None``

    Note:
      This function ignores the compare action.
    """
    if action.lower() != 'compare':
        return progress_bar(min(100, percentage), 100, prefix=action)

    return None


def calculate_parity(n):
    """Calculates and returns the parity of a number.

    The parity of a number is ``1`` if the number has an odd number of ones
    in its binary representation, otherwise ``0``.

    Args:
      n (int): the number whose parity to calculate

    Returns:
      ``1`` if the number has an odd number of ones, otherwise ``0``.

    Raises:
      ValueError: if ``n`` is less than ``0``.
    """
    if not is_natural(n):
        raise ValueError('Expected n to be a positive integer.')

    y = 0
    n = abs(n)
    while n:
        y += n & 1
        n = n >> 1
    return y & 1
